@@ -26,6 +26,7 @@
 #include <fam.h>
 #include <gio/gfilemonitor.h>
 
+#include "gfile.h"
 #include "fam-helper.h"
 
 static FAMConnection* fam_connection = NULL;
@@ -181,6 +182,22 @@ _fam_sub_startup (void)
   return TRUE;
 }
 
+void
+_fam_sub_shutdown (void)
+{
+  G_LOCK (fam_connection);
+
+  if (fam_connection != NULL) {
+    FAMClose (fam_connection);
+    g_free (fam_connection);
+    g_source_remove (fam_watch_id);
+    fam_watch_id = 0;
+    fam_connection = NULL;
+  }
+
+  G_UNLOCK (fam_connection);
+}
+
 fam_sub*
 _fam_sub_add (const gchar* pathname,
 	      gboolean directory,
@@ -191,11 +208,6 @@ _fam_sub_add (const gchar* pathname,
   if (!_fam_sub_startup ())
     return NULL;
   
-  sub = g_new0 (fam_sub, 1);
-  sub->pathname = g_strdup (pathname);
-  sub->directory = directory;
-  sub->user_data = user_data;
-  
   G_LOCK (fam_connection);
   /* We need to queue up incoming messages to avoid blocking on write
    *  if there are many monitors being canceled */
@@ -206,6 +218,11 @@ _fam_sub_add (const gchar* pathname,
     return NULL;
   }
   
+  sub = g_new0 (fam_sub, 1);
+  sub->pathname = g_strdup (pathname);
+  sub->directory = directory;
+  sub->user_data = user_data;
+  
   if (directory)
     FAMMonitorDirectory (fam_connection, pathname, &sub->request, sub);
   else
