@@ -431,6 +431,8 @@ g_local_file_get_parse_name (GFile *file)
 	      g_free (utf8_filename);
 	      utf8_filename = NULL;
 	    }
+
+	  g_free (roundtripped_filename);
 	}
     }
 
@@ -590,7 +592,7 @@ g_local_file_get_child_for_display_name (GFile        *file,
 					 const char   *display_name,
 					 GError      **error)
 {
-  GFile *parent, *new_file;
+  GFile *new_file;
   char *basename;
 
   basename = g_filename_from_utf8 (display_name, -1, NULL, NULL, NULL);
@@ -602,9 +604,7 @@ g_local_file_get_child_for_display_name (GFile        *file,
       return NULL;
     }
 
-  parent = g_file_get_parent (file);
   new_file = g_file_get_child (file, basename);
-  g_object_unref (parent);
   g_free (basename);
   
   return new_file;
@@ -779,7 +779,7 @@ get_mount_info (GFileInfo             *fs_info,
 
       mountpoint = find_mountpoint_for (path, buf.st_dev);
       if (mountpoint == NULL)
-	mountpoint = "/";
+	mountpoint = g_strdup ("/");
 
       mount = g_unix_mount_at (mountpoint, &cache_time);
       if (mount)
@@ -790,6 +790,8 @@ get_mount_info (GFileInfo             *fs_info,
 	  g_unix_mount_free (mount);
 	}
 
+      g_free (mountpoint);
+
       dev = g_new0 (dev_t, 1);
       *dev = buf.st_dev;
       
@@ -844,10 +846,10 @@ get_volume_for_path (const char *path)
   wchar_t *wpath;
   wchar_t *result;
 
-  wpath = g_utf8_to_utf16 (path, -1, NULL, &len, NULL);
-  result = g_new (wchar_t, len + 2);
+  wpath = g_utf8_to_utf16 (path, -1, NULL, NULL, NULL);
+  result = g_new (wchar_t, MAX_PATH);
 
-  if (!GetVolumePathNameW (wpath, result, len + 2))
+  if (!GetVolumePathNameW (wpath, result, MAX_PATH))
     {
       char *msg = g_win32_error_message (GetLastError ());
       g_critical ("GetVolumePathName failed: %s", msg);
@@ -1992,7 +1994,7 @@ g_local_file_make_directory (GFile         *file,
 {
   GLocalFile *local = G_LOCAL_FILE (file);
   
-  if (g_mkdir (local->filename, 0755) == -1)
+  if (g_mkdir (local->filename, 0777) == -1)
     {
       int errsv = errno;
 
