@@ -1811,13 +1811,13 @@ g_get_real_name (void)
  * </simplelist>
  * Since applications are in general <emphasis>not</emphasis> written 
  * to deal with these situations it was considered better to make 
- * g_get_homedir() not pay attention to <envar>HOME</envar> and to 
+ * g_get_home_dir() not pay attention to <envar>HOME</envar> and to 
  * return the real home directory for the user. If applications
  * want to pay attention to <envar>HOME</envar>, they can do:
  * |[
  *  const char *homedir = g_getenv ("HOME");
  *   if (!homedir)
- *      homedir = g_get_homedir (<!-- -->);
+ *      homedir = g_get_home_dir (<!-- -->);
  * ]|
  *
  * Returns: the current user's home directory
@@ -2446,6 +2446,9 @@ load_user_special_dirs (void)
  * Returns the full path of a special directory using its logical id.
  *
  * On Unix this is done using the XDG special user directories.
+ * For compatibility with existing practise, %G_USER_DIRECTORY_DESKTOP
+ * falls back to <filename>$HOME/Desktop</filename> when XDG special
+ * user directories have not been set up. 
  *
  * Depending on the platform, the user might be able to change the path
  * of the special directory without requiring the session to restart; GLib
@@ -3280,10 +3283,10 @@ glib_gettext (const gchar *str)
       _glib_gettext_initialized = TRUE;
     }
   
-  return dgettext (GETTEXT_PACKAGE, str);
+  return g_dgettext (GETTEXT_PACKAGE, str);
 }
 
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
 
 /* Binary compatibility versions. Not for newly compiled code. */
 
